package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.listeners.KeyboardFocusManager;
import eu.jonahbauer.wizard.client.libgdx.listeners.ResetErrorListener;
import eu.jonahbauer.wizard.client.libgdx.state.Menu;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

public class ConnectScreen extends MenuScreen {
    private static String uri;

    private TextButton buttonBack;
    private TextButton buttonConnect;

    private TextField uriField;

    public ConnectScreen(WizardGame game) {
        super(game);
    }
    
    private final ChangeListener listener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            if (actor == buttonBack) {
                game.getClient().execute(Menu.class, Menu::showMenuScreen);
                sfxClick();
            } else if (actor == buttonConnect) {
                try {
                    var uriString = ConnectScreen.this.uriField.getText();
                    var uri = new URI(uriString);
                    ConnectScreen.uri = uriString;
                    game.getClient().execute(Menu.class, (s, c) -> s.connect(c, uri));
                } catch (URISyntaxException e) {
                    uriField.setStyle(game.data.skin.get("error", TextField.TextFieldStyle.class));
                }

                sfxClick();
            }
        }
    };

    @Override
    public void show() {
        super.show();

        buttonBack = new TextButton(game.messages.get("menu.connect.back"), game.data.skin);
        buttonBack.setPosition(WizardGame.WIDTH * 0.275f, BUTTON_BAR_Y);
        buttonBack.addListener(listener);

        buttonConnect = new TextButton(game.messages.get("menu.connect.connect"), game.data.skin);
        buttonConnect.setPosition(WizardGame.WIDTH * 0.725f - buttonConnect.getWidth(), BUTTON_BAR_Y);
        buttonConnect.addListener(listener);

        var label = new Label(game.messages.get("menu.connect.address.label"), game.data.skin);
        label.setSize(0.4f * WizardGame.WIDTH, 64);
        label.setAlignment(Align.center);
        label.setPosition(0.5f * (WizardGame.WIDTH - label.getWidth()), 0.55f * (WizardGame.HEIGHT - label.getHeight()));

        // TODO sensible default value
        uriField = new TextField(uri != null ? uri : "wss://webdev.jonahbauer.eu/wizard/", game.data.skin);
        uriField.setMessageText(game.messages.get("menu.connect.uri.hint"));
        uriField.setSize(0.4f * WizardGame.WIDTH, 64);
        uriField.setPosition(0.5f * (WizardGame.WIDTH - uriField.getWidth()), 0.45f * (WizardGame.HEIGHT - uriField.getHeight()));
        uriField.addListener(new ResetErrorListener(game.data.skin));

        Gdx.input.setInputProcessor(game.data.stage);
        game.data.stage.addActor(buttonBack);
        game.data.stage.addActor(buttonConnect);
        game.data.stage.addActor(uriField);
        game.data.stage.addActor(label);
        game.data.stage.addCaptureListener(new KeyboardFocusManager(
                game.data.stage,
                List.of(uriField, buttonBack, buttonConnect)
        ));

        buttonBack.setName("button_back");
        buttonConnect.setName("button_connect");
        uriField.setName("uri");
    }
}
